﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Threading;
using System.Diagnostics;
using Microsoft.WindowsAPICodePack.Taskbar;
using System.Security.Cryptography;

namespace TS视频流下载合成
{
    public partial class MainForm : Form
    {
        #region
        /// <summary>
        /// 操作状态
        /// </summary>
        public OperationStatus operationStatus = OperationStatus.Normal;
        /// <summary>
        /// 当前下载索引进度
        /// </summary>
        private int indexprogress = 0;
        /// <summary>
        /// 下载耗时
        /// </summary>
        private Stopwatch downloadtime = new Stopwatch();
        /// <summary>
        /// 项目耗时
        /// </summary>
        private Stopwatch alltime = new Stopwatch();
        /// <summary>
        /// url索引替换符
        /// </summary>
        private string splirstr = "(***)";
        /// <summary>
        /// 保存根目录
        /// </summary>
        private string AppPath = null;
        /// <summary>
        /// 项目缓存保存路径
        /// </summary>
        private string AppSavePath = null;
        /// <summary>
        /// 项目缓存历史文件保存路径
        /// </summary>
        private string AppSaveHistoryPath = null;
        /// <summary>
        /// 项目缓存历史文件对象
        /// </summary>
        private ProjectItem ProjectObject = null;

        private static object projectobject_static = new object();

        private SplashForm splashForm = null;

        private DownLoadType downLoadType = DownLoadType.Normal;
        #endregion

        public MainForm()
        {
            InitializeComponent();
            this.splashForm = new SplashForm(this);
        }

        #region 项目
        /// <summary>
        /// 视频保存路径
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void savepathrootbtn_Click(object sender, EventArgs e)
        {
            try
            {
                this.savepathfbd.RootFolder = Environment.SpecialFolder.Desktop;
                this.savepathfbd.SelectedPath = "c:\\";
                this.savepathfbd.ShowNewFolderButton = true;
                this.savepathfbd.Description = "请选择保存目录";

                if (this.savepathfbd.ShowDialog() == DialogResult.OK)
                {
                    this.savepathroottb.Text = this.savepathfbd.SelectedPath;
                    this.AppPath = this.savepathroottb.Text;

                    this.BindProjects();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 选择项目信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectdirectorycom_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                this.AppSavePath = this.AppPath + @"\" + this.selectdirectorycom.SelectedItem.ToString();
                this.AppSaveHistoryPath = this.AppSavePath + @"\history.json";
                this.LoadProjectsHistory(this.AppSaveHistoryPath, false);
                this.ShowProjectsOtherInfo();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 添加项目信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void adddirectorybtndirectorybtn_Click(object sender, EventArgs e)
        {
            try
            {

                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.splashForm == null)
                {
                    this.splashForm = new SplashForm(this);
                }
                this.splashForm.ShowDialog(this);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 删除项目信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cleardirectorybtn_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult res = MessageBox.Show("确认删除?", " ", MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);    //默认焦点在"取消"按钮上
                if (res == DialogResult.OK)
                {

                    string _savepathroot = this.savepathroottb.Text.Trim();
                    if (string.IsNullOrWhiteSpace(_savepathroot))
                    {
                        MessageBox.Show("请选择保存目录根路径");
                        return;
                    }

                    if (this.selectdirectorycom.SelectedItem == null)
                    {
                        MessageBox.Show("请选择保存目录名");
                        return;
                    }

                    Directory.Delete(this.AppSavePath, true);

                    this.BindProjects();

                    this.ProjectObject = null;
                    this.AppSavePath = "";
                    this.AppSaveHistoryPath = "";

                    this.urltb.Text = "";
                    this.startindexnud.Text = "";
                    this.nowindexnud.Text = "";
                    this.endindexnud.Text = "";
                    this.isindexcb.Checked = false;
                    this.indexnud.Text = "";
                    this.isaescb.Checked = false;
                    this.aestb.Text = "";
                    this.errorlisttb.Text = "";
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 打开文件夹
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void openbtn_Click(object sender, EventArgs e)
        {
            try
            {
                #region
                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }
                #endregion

                Process proc = new Process();
                proc.StartInfo.FileName = "explorer";
                proc.StartInfo.Arguments = this.AppSavePath;
                proc.Start();
                proc.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region 操作
        /// <summary>
        /// 开始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startbtn_Click(object sender, EventArgs e)
        {
            try
            {
                this.downLoadType = DownLoadType.Normal;

                string _url = this.urltb.Text.Trim().Replace("\n", "").Replace(" ", "").Replace("\t", "").Replace("\r", "");

                #region
                if (string.IsNullOrWhiteSpace(_url))
                {
                    MessageBox.Show("请输入视频地址");
                    return;
                }
                if (_url.IndexOf(this.splirstr) < 0)
                {
                    MessageBox.Show("找不到视频地址索引替换符:" + splirstr);
                    return;
                }

                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }

                int _startindex = 0;
                if (!int.TryParse(this.startindexnud.Text, out _startindex))
                {
                    MessageBox.Show("请输入开始索引");
                    return;
                }
                int _nowindex = 0;
                if (!int.TryParse(this.nowindexnud.Text, out _nowindex))
                {
                    MessageBox.Show("请输入当前索引");
                    return;
                }
                int _endindex = 0;
                if (!int.TryParse(this.endindexnud.Text, out _endindex))
                {
                    MessageBox.Show("请输入结束索引");
                    return;
                }

                int _index = 0;
                if (this.isindexcb.Checked && !int.TryParse(this.indexnud.Text, out _index))
                {
                    MessageBox.Show("请输入索引补零");
                    return;
                }
                string _aeskey = this.aestb.Text.Trim();
                if (this.isaescb.Checked && string.IsNullOrWhiteSpace(_aeskey))
                {
                    MessageBox.Show("请输入AESkey");
                    return;
                }

                #endregion

                this.urltb.ReadOnly = true;
                this.savepathrootbtn.Enabled = false;
                this.selectdirectorycom.Enabled = false;
                this.adddirectorybtn.Enabled = false;
                this.cleardirectorybtn.Enabled = false;
                this.startindexnud.Enabled = false;
                this.nowindexnud.Enabled = false;
                this.endindexnud.Enabled = false;
                this.isindexcb.Enabled = false;
                this.indexnud.Enabled = false;
                this.isproxycb.Enabled = false;
                this.isaescb.Enabled = false;
                this.aestb.ReadOnly = true;
                this.startbtn.Enabled = false;
                this.stopbtn.Enabled = true;
                this.continuebtn.Enabled = false;
                this.canclebtn.Enabled = true;
                this.validbtn.Enabled = false;
                this.downloadbtn.Enabled = false;
                this.listdownloadbtn.Enabled = false;
                this.convertbtn.Enabled = false;


                this.allprogressva.Text = String.Format("{0}/{1}", this.startindexnud.Value, this.endindexnud.Value);
                this.errorlisttb.Text = "";

                this.operationStatus = OperationStatus.Download;

                this.alltimeva.Text = "";
                this.alltime.Restart();

                this.savehistorybtn_Click(null, null);
                TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Normal, this.Handle);
                this.ShowProjectsErrorInfo();

                this.DownTS(null, _nowindex, _endindex, this.isindexcb.Checked, _index, _url, this.AppSavePath, this.isaescb.Checked, _aeskey, this.isproxycb.Checked, this.downLoadType);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 暂停
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopbtn_Click(object sender, EventArgs e)
        {
            try
            {
                this.stopbtn.Text = "暂停中...";
                this.timeva.Text = "";

                this.alltimeva.Text = "";
                this.alltime.Stop();

                this.time.Enabled = false;

                this.operationStatus = OperationStatus.Stop;

                this.stopbtn.Enabled = false;
                this.continuebtn.Enabled = true;
                this.canclebtn.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 继续
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void continuebtn_Click(object sender, EventArgs e)
        {
            try
            {
                string _url = this.urltb.Text.Trim().Replace("\n", "").Replace(" ", "").Replace("\t", "").Replace("\r", "");

                #region
                if (string.IsNullOrWhiteSpace(_url))
                {
                    MessageBox.Show("请输入视频地址");
                    return;
                }
                if (_url.IndexOf(this.splirstr) < 0)
                {
                    MessageBox.Show("找不到视频地址索引替换符:" + splirstr);
                    return;
                }

                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }

                int _startindex = 0;
                if (!int.TryParse(this.startindexnud.Text, out _startindex))
                {
                    MessageBox.Show("请输入开始索引");
                    return;
                }
                int _nowindex = 0;
                if (!int.TryParse(this.nowindexnud.Text, out _nowindex))
                {
                    MessageBox.Show("请输入当前索引");
                    return;
                }
                int _endindex = 0;
                if (!int.TryParse(this.endindexnud.Text, out _endindex))
                {
                    MessageBox.Show("请输入结束索引");
                    return;
                }

                int _index = 0;
                if (this.isindexcb.Checked && !int.TryParse(this.indexnud.Text, out _index))
                {
                    MessageBox.Show("请输入索引补零");
                    return;
                }
                string _aeskey = this.aestb.Text.Trim();
                if (this.isaescb.Checked && string.IsNullOrWhiteSpace(_aeskey))
                {
                    MessageBox.Show("请输入AESkey");
                    return;
                }

                #endregion

                this.alltime.Start();

                this.stopbtn.Enabled = true;
                this.continuebtn.Enabled = false;
                this.canclebtn.Enabled = true;


                TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Normal, this.Handle);

                this.operationStatus = OperationStatus.Download;

                this.DownTS(null, this.indexprogress, _endindex, this.isindexcb.Checked, _index, _url, this.AppSavePath, this.isaescb.Checked, _aeskey, this.isproxycb.Checked, this.downLoadType);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 取消
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void canclebtn_Click(object sender, EventArgs e)
        {
            try
            {
                this.operationStatus = OperationStatus.Cancle;

                this.startbtn.Enabled = false;
                this.stopbtn.Enabled = false;
                this.continuebtn.Enabled = false;
                this.convertbtn.Enabled = false;
                this.canclebtn.Enabled = false;
                this.canclebtn.Text = "取消中...";

                this.alltime.Stop();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region
        /// <summary>
        /// 验证TS视频列表完整
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void validbtn_Click(object sender, EventArgs e)
        {
            try
            {
                string _url = this.urltb.Text.Trim().Replace("\n", "").Replace(" ", "").Replace("\t", "").Replace("\r", "");

                #region

                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }

                int _startindex = 0;
                if (!int.TryParse(this.startindexnud.Text, out _startindex))
                {
                    MessageBox.Show("请输入开始索引");
                    return;
                }
                int _endindex = 0;
                if (!int.TryParse(this.endindexnud.Text, out _endindex))
                {
                    MessageBox.Show("请输入结束索引");
                    return;
                }

                int _index = 0;
                if (this.isindexcb.Checked && !int.TryParse(this.indexnud.Text, out _index))
                {
                    MessageBox.Show("请输入索引补零");
                    return;
                }

                #endregion

                bool issplir = _url.IndexOf(this.splirstr) < 0 ? false : true;


                string[] files_arr = Directory.GetFiles(this.AppSavePath);
                List<string> filesList = new List<string>();
                for (int i = 0; i < files_arr.Length; i++)
                {
                    if (files_arr[i].EndsWith(".ts"))
                    {
                        filesList.Add(Path.GetFileNameWithoutExtension(files_arr[i]));
                    }
                }

                this.ProjectObject.errorlist.Clear();

                for (int i = int.Parse(this.startindexnud.Text); i < int.Parse(this.endindexnud.Text); i++)
                {
                    if (filesList.IndexOf(i.ToString().PadLeft(4, '0')) < 0)
                    {
                        string url = ((this.isindexcb.Checked && i.ToString().Length < int.Parse(this.indexnud.Text)) ? i.ToString().PadLeft(int.Parse(this.indexnud.Text), '0') : i.ToString());
                        if (issplir)
                        {
                            url = _url.Replace(this.splirstr, url);
                        }
                        this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = i.ToString().PadLeft(4, '0'), message = "下载失败", url = url });
                    }
                }
                this.ShowProjectsErrorInfo();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 验证下载信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void downloadbtn_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ProjectObject == null || this.ProjectObject.errorlist.Count < 1)
                    return;

                this.downLoadType = DownLoadType.Valid;

                #region
                string _aeskey = this.aestb.Text.Trim();
                if (this.isaescb.Checked && string.IsNullOrWhiteSpace(_aeskey))
                {
                    MessageBox.Show("请输入AESkey");
                    return;
                }
                #endregion
                List<ProjectErrorItem> ErrorList = new List<ProjectErrorItem>();
                foreach (ProjectErrorItem item in this.ProjectObject.errorlist)
                {
                    ErrorList.Add(new ProjectErrorItem() { index = item.index, message = item.message, url = item.url });
                }

                this.urltb.ReadOnly = true;
                this.savepathrootbtn.Enabled = false;
                this.selectdirectorycom.Enabled = false;
                this.adddirectorybtn.Enabled = false;
                this.cleardirectorybtn.Enabled = false;
                this.startindexnud.Enabled = false;
                this.nowindexnud.Enabled = false;
                this.endindexnud.Enabled = false;
                this.isindexcb.Enabled = false;
                this.indexnud.Enabled = false;
                this.isproxycb.Enabled = false;
                this.isaescb.Enabled = false;
                this.aestb.ReadOnly = true;
                this.startbtn.Enabled = false;
                this.stopbtn.Enabled = true;
                this.continuebtn.Enabled = false;
                this.canclebtn.Enabled = true;
                this.validbtn.Enabled = false;
                this.downloadbtn.Enabled = false;
                this.listdownloadbtn.Enabled = false;
                this.convertbtn.Enabled = false;


                this.allprogressva.Text = String.Format("{0}/{1}", 0, ErrorList.Count - 1);
                this.errorlisttb.Text = "";

                this.operationStatus = OperationStatus.Download;

                this.alltimeva.Text = "";
                this.alltime.Restart();



                this.DownTS(ErrorList, 0, ErrorList.Count - 1, false, 0, "", this.AppSavePath, this.isaescb.Checked, _aeskey, this.isproxycb.Checked, this.downLoadType);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// 列表下载
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listdownloadbtn_Click(object sender, EventArgs e)
        {
            try
            {
                this.downLoadType = DownLoadType.List;

                string _url = this.urltb.Text.Trim().Replace("\n", "").Replace(" ", "").Replace("\t", "").Replace("\r", "");
                string context = "";

                #region
                if (!File.Exists(this.AppSavePath + @"\list.txt"))
                {
                    MessageBox.Show("请添加list.txt下载列表");
                    return;
                }
                if (string.IsNullOrWhiteSpace(_url))
                {
                    MessageBox.Show("请输入视频地址");
                    return;
                }

                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存目录根路径");
                    return;
                }

                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }

                int _startindex = 0;
                if (!int.TryParse(this.startindexnud.Text, out _startindex))
                {
                    MessageBox.Show("请输入开始索引");
                    return;
                }
                int _nowindex = 0;
                if (!int.TryParse(this.nowindexnud.Text, out _nowindex))
                {
                    MessageBox.Show("请输入当前索引");
                    return;
                }
                int _endindex = 0;
                if (!int.TryParse(this.endindexnud.Text, out _endindex))
                {
                    MessageBox.Show("请输入结束索引");
                    return;
                }

                string _aeskey = this.aestb.Text.Trim();
                if (this.isaescb.Checked && string.IsNullOrWhiteSpace(_aeskey))
                {
                    MessageBox.Show("请输入AESkey");
                    return;
                }

                #endregion

                context = File.ReadAllText(this.AppSavePath + @"\list.txt");

                List<ProjectErrorItem> DownloadList = new List<ProjectErrorItem>();
                List<string> itenlist = context.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries).ToList<string>();
                int index = -1;
                foreach (string item in itenlist)
                {
                    if (item.ToLower().EndsWith(".ts"))
                    {
                        index++;
                        DownloadList.Add(new ProjectErrorItem() { index = index.ToString().PadLeft(4, '0') + "_" + Path.GetFileNameWithoutExtension(item), message = "", url = _url + item });
                    }
                }

                int startindex = 0;
                for (int i = DownloadList.Count - 1; i > -1; i--)
                {
                    if (File.Exists(this.AppSavePath + @"\" + DownloadList[i].index + ".ts"))
                    {
                        startindex = i + 1;
                        break;
                    }
                }

                this.urltb.ReadOnly = true;
                this.savepathrootbtn.Enabled = false;
                this.selectdirectorycom.Enabled = false;
                this.adddirectorybtn.Enabled = false;
                this.cleardirectorybtn.Enabled = false;
                this.startindexnud.Enabled = false;
                this.nowindexnud.Enabled = false;
                this.endindexnud.Enabled = false;
                this.isindexcb.Enabled = false;
                this.indexnud.Enabled = false;
                this.isproxycb.Enabled = false;
                this.isaescb.Enabled = false;
                this.aestb.ReadOnly = true;
                this.startbtn.Enabled = false;
                this.stopbtn.Enabled = true;
                this.continuebtn.Enabled = false;
                this.canclebtn.Enabled = true;
                this.validbtn.Enabled = false;
                this.downloadbtn.Enabled = false;
                this.listdownloadbtn.Enabled = false;
                this.convertbtn.Enabled = false;

                this.endindexnud.Value = DownloadList.Count - 1;

                this.allprogressva.Text = String.Format("{0}/{1}", startindex, DownloadList.Count - 1);
                this.errorlisttb.Text = "";

                this.operationStatus = OperationStatus.Download;

                this.alltimeva.Text = "";
                this.alltime.Restart();

                this.ShowProjectsErrorInfo();

                this.DownTS(DownloadList, startindex, DownloadList.Count - 1, false, 0, "", this.AppSavePath, this.isaescb.Checked, _aeskey, this.isproxycb.Checked, this.downLoadType);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// 保存下载信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void savehistorybtn_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ProjectObject == null)
                    return;

                this.ProjectObject.url = this.urltb.Text.Trim();
                this.ProjectObject.startindex = this.startindexnud.Text.Trim();
                this.ProjectObject.nowindex = this.nowindexnud.Text.Trim();
                this.ProjectObject.endindex = this.endindexnud.Text.Trim();
                this.ProjectObject.isindex = this.isindexcb.Checked;
                this.ProjectObject.index = this.indexnud.Text.Trim();
                this.ProjectObject.isaes = this.isaescb.Checked;
                this.ProjectObject.aes = this.aestb.Text.Trim();

                this.SaveProjectsHistory(this.AppSaveHistoryPath);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        /// <summary>
        /// 合并视频
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void convertbtn_Click(object sender, EventArgs e)
        {
            try
            {
                #region
                string _savepathroot = this.savepathroottb.Text.Trim();
                if (string.IsNullOrWhiteSpace(_savepathroot))
                {
                    MessageBox.Show("请选择保存根路径");
                    return;
                }

                string _savedirectory = this.selectdirectorycom.SelectedItem.ToString();
                if (this.selectdirectorycom.SelectedItem == null)
                {
                    MessageBox.Show("请选择保存目录名");
                    return;
                }

                #endregion

                this.operationStatus = OperationStatus.Convert;

                this.urltb.ReadOnly = true;
                this.savepathrootbtn.Enabled = false;
                this.selectdirectorycom.Enabled = false;
                this.adddirectorybtn.Enabled = false;
                this.cleardirectorybtn.Enabled = false;
                this.startindexnud.Enabled = false;
                this.nowindexnud.Enabled = false;
                this.endindexnud.Enabled = false;
                this.isindexcb.Enabled = false;
                this.indexnud.Enabled = false;
                this.isproxycb.Enabled = false;
                this.isaescb.Enabled = false;
                this.aestb.ReadOnly = true;
                this.startbtn.Enabled = false;
                this.stopbtn.Enabled = false;
                this.continuebtn.Enabled = false;
                this.canclebtn.Enabled = false;
                this.validbtn.Enabled = false;
                this.downloadbtn.Enabled = false;
                this.listdownloadbtn.Enabled = false;
                this.convertbtn.Enabled = false;

                this.allprogressva.Text = "视频合并中...";
                this.timeva.Text = "0时0分0秒";

                this.time.Enabled = true;
                this.downloadtime.Restart();

                Action<string, string> videoconvert = (string savepath, string savefilename) =>
                 {
                     List<string> fileList = Directory.GetFiles(savepath).ToList<string>().Where(a => a.ToLower().EndsWith(".ts")).ToList<string>();
                     if (fileList.Count < 1)
                     {
                         this.Invoke(new MethodInvoker(delegate() { MessageBox.Show("找不到TS文件"); }));
                         return;
                     }

                     string tmp = savepath + "\\tmp";

                     string ffmpegpath = AppDomain.CurrentDomain.BaseDirectory + "ffmpeg.exe";
                     ProcessStartInfo psi = new ProcessStartInfo(ffmpegpath);
                     psi.UseShellExecute = false;
                     psi.CreateNoWindow = true;
                     psi.WindowStyle = ProcessWindowStyle.Hidden;
                     psi.RedirectStandardError = true;
                     psi.RedirectStandardOutput = true;
                     psi.StandardOutputEncoding = Encoding.UTF8;
                     psi.StandardErrorEncoding = Encoding.UTF8;
                     Process process = null;
                     string error = "";

                     string cmd = " -i \"concat:{0}\" -acodec copy -vcodec copy -absf aac_adtstoasc  -y \"{1}\" ";
                     int maxlength = 8000;
                     List<ItemClass> tmpvideolist = new List<ItemClass>();
                     StringBuilder cmd_arg = new StringBuilder();

                     int group = 0;
                     for (int i = 0; i < fileList.Count; i++)
                     {
                         if (cmd_arg.Length > 0)
                         {
                             cmd_arg.Append("|");
                         }
                         cmd_arg.Append(fileList[i]);

                         if (cmd_arg.Length > maxlength)
                         {
                             group++;
                             cmd_arg = cmd_arg.Remove(cmd_arg.Length - fileList[i].Length, fileList[i].Length);
                             tmpvideolist.Add(new ItemClass() { tmpfile = tmp + "\\" + group.ToString().PadLeft(4, '0') + ".mp4", arg = cmd_arg.ToString() });

                             cmd_arg = new StringBuilder();
                             if (cmd_arg.Length > 0)
                             {
                                 cmd_arg.Append("|");
                             }
                             cmd_arg.Append(fileList[i]);
                         }
                     }


                     if (tmpvideolist.Count < 1)
                     {
                         psi.Arguments = String.Format(cmd, cmd_arg, savepath + "\\" + savefilename + ".mp4");
                         process = Process.Start(psi);
                         error = process.StandardError.ReadToEnd();
                         process.WaitForExit();
                     }
                     else
                     {

                         if (!Directory.Exists(tmp))
                             Directory.CreateDirectory(tmp);


                         string tmpfile = tmp + "\\filelist.txt";
                         FileStream fs = new FileStream(tmpfile, FileMode.Create);

                         for (int i = 0; i < tmpvideolist.Count; i++)
                         {
                             psi.Arguments = String.Format(cmd, tmpvideolist[i].arg, tmpvideolist[i].tmpfile);
                             process = Process.Start(psi);
                             error = process.StandardError.ReadToEnd();
                             process.WaitForExit();

                             byte[] data = System.Text.Encoding.Default.GetBytes("file '" + tmpvideolist[i].tmpfile + "'\r\n");
                             fs.Write(data, 0, data.Length);
                         }

                         fs.Flush();
                         fs.Close();

                         psi.Arguments = String.Format(" -f concat -safe 0 -i \"{0}\" -c copy  -y \"{1}\"", tmpfile, savepath + "\\" + savefilename + ".mp4");
                         process = Process.Start(psi);
                         error = process.StandardError.ReadToEnd();
                         process.WaitForExit();

                         if (Directory.Exists(tmp))
                         {
                             Directory.Delete(tmp, true);
                         }

                     }



                     this.Invoke(new MethodInvoker(delegate()
                    {
                        this.operationStatus = OperationStatus.Normal;

                        this.allprogressva.Text = "完成";
                        this.timeva.Text = "";

                        this.time.Enabled = false;

                        this.urltb.ReadOnly = false;
                        this.savepathrootbtn.Enabled = true;
                        this.selectdirectorycom.Enabled = true;
                        this.adddirectorybtn.Enabled = true;
                        this.cleardirectorybtn.Enabled = true;
                        this.startindexnud.Enabled = true;
                        this.nowindexnud.Enabled = true;
                        this.endindexnud.Enabled = true;
                        this.isindexcb.Enabled = true;
                        this.indexnud.Enabled = true;
                        this.isproxycb.Enabled = true;
                        this.isaescb.Enabled = true;
                        this.aestb.ReadOnly = false;
                        this.startbtn.Enabled = true;
                        this.stopbtn.Enabled = true;
                        this.continuebtn.Enabled = true;
                        this.canclebtn.Enabled = true;
                        this.validbtn.Enabled = true;
                        this.downloadbtn.Enabled = true;
                        this.listdownloadbtn.Enabled = true;
                        this.convertbtn.Enabled = true;
                    }));

                 };

                AsyncCallback callFun = (result) => { ((Action<string, string>)result.AsyncState).EndInvoke(result); };
                videoconvert.BeginInvoke(this.AppSavePath, _savedirectory, callFun, videoconvert);

            }
            catch (Exception ex)
            {
                this.operationStatus = OperationStatus.Normal;

                this.allprogressva.Text = "视频合并失败";

                this.time.Enabled = false;

                this.urltb.ReadOnly = false;
                this.savepathrootbtn.Enabled = true;
                this.selectdirectorycom.Enabled = true;
                this.adddirectorybtn.Enabled = true;
                this.cleardirectorybtn.Enabled = true;
                this.startindexnud.Enabled = true;
                this.nowindexnud.Enabled = true;
                this.endindexnud.Enabled = true;
                this.isindexcb.Enabled = true;
                this.indexnud.Enabled = true;
                this.isproxycb.Enabled = true;
                this.isaescb.Enabled = true;
                this.aestb.ReadOnly = false;
                this.startbtn.Enabled = true;
                this.stopbtn.Enabled = true;
                this.continuebtn.Enabled = true;
                this.canclebtn.Enabled = true;
                this.validbtn.Enabled = true;
                this.downloadbtn.Enabled = true;
                this.listdownloadbtn.Enabled = true;
                this.convertbtn.Enabled = true;
            }
        }
        /// <summary>
        /// 清除错误信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearerrorbtn_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.ProjectObject.errorlist != null)
                {
                    this.ProjectObject.errorlist.Clear();
                    this.ShowProjectsErrorInfo();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        #endregion

        /// <summary>
        /// 定时更新耗时
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void time_Tick(object sender, EventArgs e)
        {
            this.timeva.Text = String.Format("{0}时{1}分{2}秒", this.downloadtime.Elapsed.Hours, this.downloadtime.Elapsed.Minutes, this.downloadtime.Elapsed.Seconds);
            this.alltimeva.Text = String.Format("{0}时{1}分{2}秒", this.alltime.Elapsed.Hours, this.alltime.Elapsed.Minutes, this.alltime.Elapsed.Seconds);
        }

        /// <summary>
        /// 下载TS流
        /// </summary>
        /// <param name="_errorlist">下载失败列表(_errorlist)为null时正常下载</param>
        /// <param name="_start">开始索引</param>
        /// <param name="_end">结束索引</param>
        /// <param name="_isindex">索引是否补零</param>
        /// <param name="_indexnum">索引补零位数</param>
        /// <param name="_url">下载TS地址</param>
        /// <param name="_savepath">TS保存路径</param>
        /// <param name="_isaes">是否aes解密</param>
        /// <param name="_aeskey">16为aeskey</param>
        /// <param name="_isproxy">禁用本地代理</param>
        /// <param name="_downLoadType">下载类型</param>
        private void DownTS(List<ProjectErrorItem> _errorlist, int _start, int _end, bool _isindex, int _indexnum, string _url, string _savepath, bool _isaes, string _aeskey, bool _isproxy, DownLoadType _downLoadType)
        {
            if (!Directory.Exists(_savepath))
                Directory.CreateDirectory(_savepath);

            Action<List<ProjectErrorItem>, int, int, bool, int, string, string, bool, string, bool, DownLoadType> videodownload = (List<ProjectErrorItem> errorlist, int start, int end, bool isindex, int indexnum, string url, string savepath, bool isaes, string aeskey, bool isproxy, DownLoadType downLoadType) =>
            {
                int _i = (downLoadType == DownLoadType.Normal || downLoadType == DownLoadType.List) ? start : 0;
                for (int i = _i; i <= end; i++)
                {
                    this.indexprogress = i;

                    OperationStatus status = IsStopOrCancle();
                    if (status == OperationStatus.Stop || status == OperationStatus.Cancle)
                    {
                        return;
                    }

                    this.Invoke(new MethodInvoker(delegate()
                    {
                        if (this.time.Enabled == false)
                        {
                            this.time.Enabled = true;
                        }

                        this.allprogressva.Text = String.Format("{0}/{1}", i, end);
                        this.progressva.Text = "00.00%";
                        this.sizeva.Text = "";
                        this.nowindexnud.Value = indexprogress;

                        TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Normal, this.Handle);
                        TaskbarManager.Instance.SetProgressValue(i, end, this.Handle);
                    }));

                    this.downloadtime.Restart();

                    string url_tmp = url.Replace(this.splirstr, ((isindex && i.ToString().Length < indexnum) ? i.ToString().PadLeft(indexnum, '0') : i.ToString()));
                    if (downLoadType == DownLoadType.Valid || downLoadType == DownLoadType.List)
                        url_tmp = errorlist[i].url;

                    string savepathfilename = savepath + "\\" + ((downLoadType == DownLoadType.Valid || downLoadType == DownLoadType.List) ? errorlist[i].index : (i.ToString().PadLeft(4, '0'))) + ".ts";
                    string savepathfilenametmp = savepathfilename + ".tmp";

                    bool success = this.HttpDownloadFile(this, i.ToString().PadLeft(4, '0'), url_tmp, savepathfilenametmp, _isproxy);
                    this.savehistorybtn_Click(null, null);
                    if (success)
                    {
                        if (isaes)
                        {
                            FileStream fs_tmp = null;
                            FileStream fs_save = null;

                            try
                            {
                                fs_tmp = new FileStream(savepathfilenametmp, FileMode.Open, FileAccess.Read);
                                byte[] buffur = new byte[fs_tmp.Length];
                                fs_tmp.Read(buffur, 0, (int)fs_tmp.Length);
                                byte[] resultbuffur = this.AESDecrypt(buffur, aeskey, aeskey);

                                fs_save = new FileStream(savepathfilename, FileMode.Create, FileAccess.ReadWrite);
                                fs_save.Write(resultbuffur, 0, resultbuffur.Length);

                                fs_tmp.Dispose();
                                fs_save.Dispose();

                                File.Delete(savepathfilenametmp);
                            }
                            catch (Exception ex)
                            {
                                this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = i.ToString().PadLeft(4, '0'), message = ex.Message, url = url });
                                this.Invoke(new MethodInvoker(delegate() { this.ShowProjectsErrorInfo(); }));
                            }
                            finally
                            {
                                if (fs_tmp != null)
                                {
                                    fs_tmp.Dispose();
                                }
                                if (fs_save != null)
                                {
                                    fs_save.Dispose();
                                }
                            }
                        }
                        else
                        {
                            if (File.Exists(savepathfilename))
                            {
                                File.Delete(savepathfilename);
                            }
                            File.Move(savepathfilenametmp, savepathfilename);
                        }
                    }

                    this.Invoke(new MethodInvoker(delegate() { this.timeva.Text = "0时0分0秒"; }));
                }

                this.operationStatus = OperationStatus.Normal;


                this.Invoke(new MethodInvoker(delegate()
                {
                    this.allprogressva.Text = "完成";
                    this.timeva.Text = "";

                    this.time.Enabled = false;

                    this.urltb.ReadOnly = false;
                    this.savepathrootbtn.Enabled = true;
                    this.selectdirectorycom.Enabled = true;
                    this.adddirectorybtn.Enabled = true;
                    this.cleardirectorybtn.Enabled = true;
                    this.startindexnud.Enabled = true;
                    this.nowindexnud.Enabled = true;
                    this.endindexnud.Enabled = true;
                    this.isindexcb.Enabled = true;
                    this.indexnud.Enabled = true;
                    this.isproxycb.Enabled = true;
                    this.isaescb.Enabled = true;
                    this.aestb.ReadOnly = false;
                    this.startbtn.Enabled = true;
                    this.stopbtn.Enabled = false;
                    this.continuebtn.Enabled = false;
                    this.canclebtn.Enabled = false;
                    this.validbtn.Enabled = true;
                    this.downloadbtn.Enabled = true;
                    this.listdownloadbtn.Enabled = true;
                    this.convertbtn.Enabled = true;

                    if (this.closewincb.Checked)
                    {
                        this.savehistorybtn_Click(null, null);

                        System.Diagnostics.Process myProcess = new System.Diagnostics.Process();
                        myProcess.StartInfo.FileName = "cmd.exe";
                        myProcess.StartInfo.UseShellExecute = false;
                        myProcess.StartInfo.RedirectStandardInput = true;
                        myProcess.StartInfo.RedirectStandardOutput = true;
                        myProcess.StartInfo.RedirectStandardError = true;
                        myProcess.StartInfo.CreateNoWindow = true;
                        myProcess.Start();
                        myProcess.StandardInput.WriteLine("shutdown -s -t 0");//执行关机命令
                    }

                }));

            };

            AsyncCallback callFun = (result) => { ((Action<List<ProjectErrorItem>, int, int, bool, int, string, string, bool, string, bool, DownLoadType>)result.AsyncState).EndInvoke(result); };
            videodownload.BeginInvoke(_errorlist, _start, _end, _isindex, _indexnum, _url, _savepath, this.isaescb.Checked, _aeskey, this.isproxycb.Checked, _downLoadType, callFun, videodownload);

        }

        /// <summary>
        /// 是否暂停或取消动作
        /// </summary>
        /// <returns></returns>
        private OperationStatus IsStopOrCancle()
        {
            if (this.operationStatus == OperationStatus.Stop)
            {
                this.Invoke(new MethodInvoker(delegate()
                {
                    this.stopbtn.Text = "暂停";

                    this.time.Enabled = false;

                    TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Paused, this.Handle);
                    this.progressva.Text = "";
                    this.sizeva.Text = "";
                }));
                return OperationStatus.Stop;
            }
            else if (this.operationStatus == OperationStatus.Cancle)
            {
                this.Invoke(new MethodInvoker(delegate()
                {
                    this.allprogressva.Text = "";
                    this.timeva.Text = "";
                    this.canclebtn.Text = "取消";
                    this.sizeva.Text = "";

                    this.time.Enabled = false;

                    this.urltb.ReadOnly = false;
                    this.savepathrootbtn.Enabled = true;
                    this.selectdirectorycom.Enabled = true;
                    this.adddirectorybtn.Enabled = true;
                    this.cleardirectorybtn.Enabled = true;
                    this.startindexnud.Enabled = true;
                    this.nowindexnud.Enabled = true;
                    this.endindexnud.Enabled = true;
                    this.isindexcb.Enabled = true;
                    this.indexnud.Enabled = true;
                    this.isproxycb.Enabled = true;
                    this.isaescb.Enabled = true;
                    this.aestb.ReadOnly = false;
                    this.startbtn.Enabled = true;
                    this.stopbtn.Enabled = false;
                    this.continuebtn.Enabled = false;
                    this.canclebtn.Enabled = false;
                    this.validbtn.Enabled = true;
                    this.listdownloadbtn.Enabled = true;
                    this.downloadbtn.Enabled = true;
                    this.convertbtn.Enabled = true;

                    this.progressva.Text = "";

                    TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Paused, this.Handle);
                }));
                return OperationStatus.Cancle;
            }
            return OperationStatus.Normal;
        }

        /// <summary>
        /// GET下载文件
        /// </summary>
        /// <param name="own">窗体</param>
        /// <param name="index">保存索引</param>
        /// <param name="url">请求地址</param>
        /// <param name="filepath">文件保存完整物理路径</param>
        /// <param name="isproxy">禁用本地代理</param>
        private bool HttpDownloadFile(Form own, string index, string url, string filepath, bool isproxy)
        {
            Stream fileStream = null;
            Stream spStream = null;
            HttpWebResponse response = null;

            int count = 0;

        down:
            try
            {
                if (url.StartsWith("https:"))
                {
                    ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3
                                        | SecurityProtocolType.Tls
                                        | (SecurityProtocolType)0x300 //Tls11
                                        | (SecurityProtocolType)0xC00; //Tls12
                }

                HttpWebRequest webReq = (HttpWebRequest)WebRequest.Create(new Uri(url));
                webReq.Method = "GET";
                if (isproxy)
                {
                    webReq.Proxy = null;
                }

                response = (HttpWebResponse)webReq.GetResponse();
                spStream = response.GetResponseStream();

                own.Invoke(new MethodInvoker(delegate() { this.sizeva.Text = GetFileSize(response.ContentLength); }));

                fileStream = new FileStream(filepath, FileMode.Create);
                byte[] bArr = new byte[1024];
                int size = spStream.Read(bArr, 0, (int)bArr.Length);
                long size_now = 0;
                while (size > 0)
                {

                    OperationStatus status = IsStopOrCancle();
                    if (status == OperationStatus.Stop)
                    {
                        if (fileStream != null)
                            fileStream.Close();
                        return true;
                    }
                    else if (status == OperationStatus.Cancle)
                    {
                        if (fileStream != null)
                            fileStream.Close();

                        size_now = 0;
                        File.Delete(filepath);
                        return false;
                    }
                    else
                    {
                        size_now += size;
                        own.Invoke(new MethodInvoker(delegate() { this.progressva.Text = String.Format("{0}%", (((float)size_now / (float)response.ContentLength) * 100).ToString("N2")); }));

                        fileStream.Write(bArr, 0, size);
                        size = spStream.Read(bArr, 0, (int)bArr.Length);
                    }
                }

                if (size_now == response.ContentLength)
                {
                    return true;
                }
                else
                {
                    if (fileStream != null)
                        fileStream.Close();
                    File.Delete(filepath);

                    this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = index, message = "[链接失败] ", url = url });
                    own.Invoke(new MethodInvoker(delegate()
                    {
                        TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Error, this.Handle);
                        this.ShowProjectsErrorInfo();
                    }));
                    return false;
                }

            }
            catch (WebException ex)
            {
                if (ex.Status == WebExceptionStatus.Timeout)
                {
                    if (fileStream != null)
                        fileStream.Close();
                    if (spStream != null)
                        spStream.Close();
                    if (response != null)
                        response.Close();
                    if (count > 3)
                    {
                        count++;
                        goto down;
                    }
                    else
                    {
                        File.Delete(filepath);

                        this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = index, message = "[链接失败] ", url = url });
                        own.Invoke(new MethodInvoker(delegate()
                        {
                            TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Error, this.Handle);
                            this.ShowProjectsErrorInfo();
                        }));
                    }
                }
                else
                {
                    if (fileStream != null)
                        fileStream.Close();
                    File.Delete(filepath);

                    this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = index, message = ex.Message, url = url });
                    own.Invoke(new MethodInvoker(delegate()
                    {
                        TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Error, this.Handle);
                        this.ShowProjectsErrorInfo();
                    }));
                }

                return false;
            }
            catch (Exception ex)
            {
                if (fileStream != null)
                    fileStream.Close();
                File.Delete(filepath);

                this.ProjectObject.errorlist.Add(new ProjectErrorItem() { index = index, message = ex.Message, url = url });
                own.Invoke(new MethodInvoker(delegate()
                {
                    TaskbarManager.Instance.SetProgressState(TaskbarProgressBarState.Error, this.Handle);
                    this.ShowProjectsErrorInfo();
                }));

                return false;
            }
            finally
            {
                if (fileStream != null)
                    fileStream.Close();
                if (spStream != null)
                    spStream.Close();
                if (response != null)
                    response.Close();
            }

        }

        /// <summary>
        /// AES解密
        /// </summary>
        /// <param name="data">被解密的密文</param>
        /// <param name="key">密钥</param>
        /// <param name="vector">向量</param> 
        /// <param name="cipherMode">对称算法的运算模式</param>  
        /// <param name="paddingMode">对称算法中使用的填充模式</param> 
        /// <returns>返回指定编码明文</returns>
        public byte[] AESDecrypt(byte[] dataBytes, string key, string vector, CipherMode cipherMode = CipherMode.CBC, PaddingMode paddingMode = PaddingMode.PKCS7)
        {
            byte[] cryptoBytes = null;
            try
            {
                using (AesManaged aes = new AesManaged())
                {
                    aes.Key = Encoding.UTF8.GetBytes(key);
                    aes.IV = Encoding.UTF8.GetBytes(vector);
                    aes.Mode = cipherMode;
                    aes.Padding = paddingMode;
                    using (MemoryStream ms = new MemoryStream(dataBytes))
                    {
                        using (CryptoStream cs = new CryptoStream(ms, aes.CreateDecryptor(), CryptoStreamMode.Read))
                        {
                            using (MemoryStream originalMemory = new MemoryStream())
                            {
                                byte[] bufferBytes = new byte[1024];
                                int bytesRead = 0;
                                while ((bytesRead = cs.Read(bufferBytes, 0, bufferBytes.Length)) > 0)
                                {
                                    originalMemory.Write(bufferBytes, 0, bytesRead);
                                }
                                cryptoBytes = originalMemory.ToArray();
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                cryptoBytes = null;
            }
            return cryptoBytes;
        }

        /// <summary>
        /// 获取所有项目名称
        /// </summary>
        /// <returns></returns>
        public List<string> GetProjects()
        {
            List<string> result = new List<string>();
            string[] arr = Directory.GetDirectories(this.AppPath);
            for (int i = 0; i < arr.Length; i++)
            {
                result.Add(new DirectoryInfo(arr[i]).Name);
            }
            return result;
        }

        /// <summary>
        /// 绑定项目名称列表
        /// </summary>
        public void BindProjects()
        {
            this.selectdirectorycom.Items.Clear();
            foreach (string item in this.GetProjects())
            {
                this.selectdirectorycom.Items.Add(item);
            }
        }

        /// <summary>
        /// 保存项目历史信息
        /// </summary>
        /// <param name="projectsHistoryPath"></param>
        public void SaveProjectsHistory(string projectsHistoryPath)
        {
            lock (projectobject_static)
            {
                string dir = Path.GetDirectoryName(projectsHistoryPath);
                if (!Directory.Exists(dir))
                {
                    Directory.CreateDirectory(dir);
                }
                File.WriteAllText(projectsHistoryPath, Newtonsoft.Json.JsonConvert.SerializeObject(this.ProjectObject), Encoding.UTF8);
            }
        }

        /// <summary>
        /// 加载项目历史信息
        /// </summary>
        /// <param name="projectsHistoryPath"></param>
        /// <param name="updateroot"></param>
        private void LoadProjectsHistory(string projectsHistoryPath, bool updateroot)
        {
            string _savepathroot = this.savepathroottb.Text.Trim();
            if (!File.Exists(projectsHistoryPath))
            {
                string dir = Path.GetDirectoryName(projectsHistoryPath);
                if (!Directory.Exists(dir))
                {
                    Directory.CreateDirectory(dir);
                }
                File.WriteAllText(projectsHistoryPath, Newtonsoft.Json.JsonConvert.SerializeObject(new ProjectItem()), Encoding.UTF8);
            }
            this.ProjectObject = Newtonsoft.Json.JsonConvert.DeserializeObject<ProjectItem>(File.ReadAllText(projectsHistoryPath, Encoding.UTF8));
            if (!updateroot)
            {
                ProjectObject.savepathroot = _savepathroot;
            }
        }

        /// <summary>
        /// 显示项目信息
        /// </summary>
        private void ShowProjectsOtherInfo()
        {
            this.urltb.Text = this.ProjectObject.url;
            this.savepathroottb.Text = this.ProjectObject.savepathroot;
            this.startindexnud.Text = this.ProjectObject.startindex;
            this.nowindexnud.Text = this.ProjectObject.nowindex;
            this.endindexnud.Text = this.ProjectObject.endindex;
            this.isindexcb.Checked = this.ProjectObject.isindex;
            this.indexnud.Text = this.ProjectObject.index;
            this.isaescb.Checked = this.ProjectObject.isaes;
            this.aestb.Text = this.ProjectObject.aes;

            this.ShowProjectsErrorInfo();
        }

        /// <summary>
        /// 显示项目错误信息列表
        /// </summary>
        public void ShowProjectsErrorInfo()
        {
            StringBuilder error_sb = new StringBuilder();
            for (int i = 0; i < this.ProjectObject.errorlist.Count; i++)
            {
                error_sb.Append(String.Format("[{0}]{1}\r\n", this.ProjectObject.errorlist[i].message, this.ProjectObject.errorlist[i].url));
            }
            this.errorlisttb.Text = error_sb.ToString();
            this.errorlistva.Text = String.Format("({0})", this.ProjectObject.errorlist.Count);
        }

        /// <summary>
        /// 确认事件
        /// </summary>
        public void Confirm(string name)
        {
            if (!string.IsNullOrWhiteSpace(name) && this.GetProjects().IndexOf(name) == -1)
            {
                string dir = this.AppPath + @"\" + name;
                if (!Directory.Exists(dir))
                {
                    Directory.CreateDirectory(dir);
                }
                this.BindProjects();

                this.selectdirectorycom.SelectedIndex = this.GetProjects().IndexOf(name);
            }

        }

        /// <summary>
        /// 计算文件大小函数(保留两位小数),Size为字节大小
        /// </summary>
        /// <param name="size">初始文件大小</param>
        /// <returns></returns>
        public static string GetFileSize(long size)
        {
            var num = 1024.00; //byte

            if (size < num)
                return size + "B";
            if (size < Math.Pow(num, 2))
                return (size / num).ToString("f2") + "K"; //kb
            if (size < Math.Pow(num, 3))
                return (size / Math.Pow(num, 2)).ToString("f2") + "M"; //M
            if (size < Math.Pow(num, 4))
                return (size / Math.Pow(num, 3)).ToString("f2") + "G"; //G


            return (size / Math.Pow(num, 4)).ToString("f2") + "T"; //T
        }
        /// <summary>
        ///  TS临时文件
        /// </summary>
        public class ItemClass
        {
            /// <summary>
            /// 临时文件路径
            /// </summary>
            public string tmpfile;
            /// <summary>
            /// 临时文件合成命令
            /// </summary>
            public string arg;

        }
        /// <summary>
        /// 操作状态
        /// </summary>
        public enum OperationStatus
        {
            /// <summary>
            /// 正常
            /// </summary>
            Normal,
            /// <summary>
            /// 下载
            /// </summary>
            Download,
            /// <summary>
            /// 取消
            /// </summary>
            Cancle,
            /// <summary>
            /// 暂停
            /// </summary>
            Stop,
            /// <summary>
            /// 合并
            /// </summary>
            Convert
        }

        /// <summary>
        /// 下载类型
        /// </summary>
        public enum DownLoadType
        {

            /// <summary>
            /// 普通
            /// </summary>
            Normal,
            /// <summary>
            /// 验证
            /// </summary>
            Valid,
            /// <summary>
            /// 列表
            /// </summary>
            List,
        }

        public class ProjectItem
        {
            public string url { get; set; }
            public string savepathroot { get; set; }
            public string startindex { get; set; }
            public string nowindex { get; set; }
            public string endindex { get; set; }
            public bool isindex { get; set; }
            public string index { get; set; }
            public bool isaes { get; set; }
            public string aes { get; set; }

            public List<ProjectErrorItem> errorlist = new List<ProjectErrorItem>();
        }

        public class ProjectErrorItem
        {
            public string index { get; set; }
            public string message { get; set; }
            public string url { get; set; }
        }



    }
}
